/**
 * Used to implement the CallableTask stack.
 *
 * @version 1.0 8/25/02
 *
 * @author Parallax, Inc.
 */

package stamp.util.os;

import stamp.util.*;

public class CallStackEntry {
  protected CallStackEntry nextEntry ;
  protected int returnState ;
  protected Object parameter ;

  protected static CallStackEntry freeList = null ;

  protected CallStackEntry () {
  }

  /**
   * Return entry to free list
   */
  void free () {
    nextEntry = freeList ;
    freeList = this ;
  }


  /**
   * Return an unused stack entry.
   * A new entry is created if the free list is empty.
   *
   * @param returnState returnState to be saved in the new entry
   * @param nextEntry nextEntry to be saved in the new entry
   * @param parameter parameter to be saved in the new entry
   *
   * @returns unused stack entry
   */
  static CallStackEntry getEntry
      ( int returnState
      , CallStackEntry nextEntry
      , Object parameter) {
    CallStackEntry result ;

    if ( freeList == null ) {
      result = new CallStackEntry () ;
    } else {
      result = freeList ;
      freeList = result.nextEntry ;
    }

    // Add entry to stack list
    result.returnState = returnState ;
    result.nextEntry = nextEntry ;
    result.parameter = parameter ;

    return result ;
  }
}