/*
 * Co-operative, multitasking state machine operating system
 * <p>
 * Tasks are a subclass of Event so a task can be resumed
 * by causing an Event.
 *
 * @author Parallax, Inc.
 *
 * @version 1.0 8/21/02
 */

package stamp.util.os;

import stamp.util.*;
import stamp.core.*;

/**
 * Interface for indicating an event occurred.
 */

 public class Event {
  static final public Event nullEvent = new Event () ;

  /**
   * Create null event. Use Event.nullEvent instead.
   */
  protected Event () {
  }

  /**
   * Check if event reference is null.
   *
   * @param event event to check
   *
   * @returns nullEvent if null, otherwise event
   */
  static public Event checkEvent ( Event event ) {
    return ( event == null ) ? nullEvent : event ;
  }

  /**
   * Cause an event to occur.
   */
  public void notify() {
    notify(null);
  }

  /**
   * Cause an event to occur.
   */
  public void notify( Object object ) {
  }

  /**
   * Get event name.
   *
   * @return name of event.
   */
  public String name () {
    return "Event" ;
  }
}