package stamp.util.os;
import stamp.core.*;

/**
 * This class extends the SimpleTask class to keep flashing a led.
 * The led connects, via a resistor, with its anode to +5V.
 * By passing this object to classes that have methods that require a significant
 * amount of time, such methods can call execute() to keep performing this task.
 *
 * Revision History:
 * 02/07/2005: v1.0: Initial release.
 * </pre><p>
 * @author Peter Verkaik (peterverkaik@boselectro.nl)
 * version 1.0 February 7, 2005
 */

public class FlashLed extends SimpleTask {

  private int ledPin;
  private ScaledTimer16 ledTimer;
  public boolean led=false; //true if led is on

  /**
   * Constructor.
   *
   * @param buttonPin;
   * @param ledPin;
   * @param ledTimer;
   */
  public FlashLed(int ledPin, ScaledTimer16 ledTimer) {
    super();
    this.ledPin = ledPin;
    this.ledTimer = ledTimer;
    CPU.setInput(ledPin);
  }

  /**
   * Execute simple task.
   */
  public void execute() {
    if (ledTimer != null) {
      if (ledTimer.timeout()) {
        ledTimer.mark();
        if (!led) {
          CPU.writePin(ledPin,false);
        }
        else {
          CPU.setInput(ledPin);
        }
        led = !led;
      }
    }
  }

}