/**
 * This type of task can be interrupted using the Interrupt() method.
 *
 * It changes the state of the task to Interrupt and saves the current
 * state. The task can then resume using lastState.
 *
 *  public void execute (int state) {
 *   switch ( state ) {
 *   case interrupt:
 *     // do something here with interruptValue
 *
 *     nextState ( lastState ) ;
 *     break ;
 *
 * @author William Wong
 *
 * @version 3.0 8/23/02 execute() parameters removed
 * @version 2.0 7/23/02 state variable to execute()
 * @version 1.0 7/21/02
 */

package stamp.util.os;

import stamp.util.*;

public abstract class InterruptTask extends Task {
  public int interruptState = terminate ;
  public int interruptValue = 0 ;

  /**
   * Used to interrupt a task. If multiple interrupts are made before the
   * task enters the interrupt state then only the first interrupt will be
   * recognized. All others will be ignored.
   *
   * @param interruptValue value given to task to determine the cause of the interrupt
   *
   * @returns true if the interrupt will be processed
   */
  public boolean interrupt ( int interruptValue ) {
    if ( state != interrupt ) {
      this.interruptValue = interruptValue ;
      interruptState = state ;
      state = interrupt ;
      return true ;
    } else
      return false ;
  }

  /**
   * Resume state prior to interrupt. This should only be called from
   * the interrupt state.
   */
  public void resumeInterrupt () {
    state = interruptState ;
  }

  /**
   * Used to interrupt a task. If multiple interrupts are made before the
   * task enters the interrupt state then only the first interrupt will be
   * recognized. All others will be ignored.
   *
   * @returns true if the interrupt will be processed
   */
  public boolean interrupt () {
    return interrupt(0);
  }
}