package stamp.util.os;

import java.io.* ;
import stamp.core.* ;


/**
 * Display multitasking status
 * <p>
 * Shows active tasks and free memory.
 * This is a class object only.
 *
 * @version 1.0 8/23/02
 * @author Parallax Inc.
 */

public class SemaphoreStatus extends ShowStatus {
  protected SemaphoreStatus () {
    // Prevents creation of object
  }

  /**
   * Print task status including semaphore information
   */
  public static void printTaskStatus ( Task task ) {
    // Print general status
    TaskStatus.printTaskStatus ( task ) ;

    // Print acquired semaphores
    for ( Semaphore semaphore = Semaphore.semaphoreList
        ; semaphore != null
        ; semaphore = semaphore.nextSemaphore
        ) {
      if ( semaphore.acquiredTask == task ) {
        print ( " Acquired " ) ;
        print ( semaphore ) ;
        println () ;
      }
    }

    // Print semaphore waiting status
    for ( Semaphore semaphore = Semaphore.semaphoreList
        ; semaphore != null
        ; semaphore = semaphore.nextSemaphore
        ) {
      for ( Task waitingTask = semaphore.waitingTaskList
          ; waitingTask != null
          ; waitingTask = waitingTask.nextTask
          ) {
        if ( waitingTask == task ) {
          print ( " Waiting for " ) ;
          print ( semaphore ) ;
          println () ;
        }
      }
    }
  }

  /**
   * Generate a random number between 0 and limit
   *
   * @param stream output stream
   */
  public static void show () {
    println ( "== Semaphore Status ==" ) ;

    for ( Semaphore semaphore = Semaphore.semaphoreList
        ; semaphore != null
        ; semaphore = semaphore.nextSemaphore
        ) {
      print ( semaphore ) ;
      if ( semaphore.ready ()) {
        println ( "ready" ) ;
      } else {
        print ( "acquired by " ) ;
        print ( semaphore.acquiredTask) ;
        println () ;

        if ( semaphore.acquiredTask.state == Task.stopped ) {
          println ( " ** Error: Task stopped **" ) ;
        }

        println ( " Waiting tasks" ) ;

        for ( Task task = semaphore.waitingTaskList
            ; task != null
            ; task = task.nextTask
            ) {
          print ( "  " ) ;
          print ( task ) ;
          println () ;
        }
      }
    }

    println () ;
  }
}