/*
 * Copyright  1999-2002 Celsius Research Ltd. All rights reserved.
 * Do not modify this file.
 */

package stamp.util.os;

import stamp.util.*;
import java.util.*;

/**
 * Co-operative RTOS
 *
 * This very basic RTOS does round robin scheduling of tasks.
 *
 * @author Bruce Wilson
 * @author Chris Waters
 */

public class TaskManager {

  // States
  public final static int STATE_STOPPED = 0;
  public final static int STATE_READY = 1;
  public final static int STATE_WAITING = 2;
  public final static int PUBLIC_STATES = STATE_WAITING+1;

  /**
   * Turns on debug mode.
   */
  public boolean debug;

  LinkedList taskList = new LinkedList();

  public Task currentTask = null;

  /**
   * Add a task to ready list
   */
  public void addTask(Task aNewTask) {
    taskList.addItem(aNewTask);
  }

  /**
   * This method will run forever executing each task in the readyTask list
   * in a round robin fashion
   */
  public void run() {
    int cycleCount = 0;

    if ( debug )
      System.out.println("RTOS Running...");

    currentTask = (Task)taskList.getFirst(); // Set to First Task

    while (true) { // Run forever.
      cycleCount++;
      try {
        currentTask = (Task)taskList.getNextLoop(currentTask); // Get Next task
        switch(currentTask.state) {
        case STATE_READY:
          currentTask.execute();
          break;

        case STATE_WAITING:
          currentTask.wait();
          break;
        }
      }
      catch (NoSuchElementException d) {    // No tasks found.
        currentTask = null;
      }
    }
  }
}