package stamp.util.os;

import java.io.* ;
import stamp.core.* ;


/**
 * Display multitasking status
 * <p>
 * Shows active tasks and free memory.
 * This is a class object only.
 *
 * @version 2.0 2/14/03   Task.taskStatus support
 * @version 1.0 8/23/02   initial version
 * @author Parallax Inc.
 */

public class TaskStatus extends ShowStatus {
  protected TaskStatus () {
    // Prevents creation of object
  }

  protected static void printTaskStatus ( int taskStatus ) {
    switch ( taskStatus ) {
    case Task.taskRunning:
      print ( "Running   " ) ;
      break ;

    case Task.taskSuspended:
      print ( "Suspended " ) ;
      break ;

    case Task.taskSemaphoreWait:
      print ( "Semaphore " ) ;
      break ;

    default:
      print ( "-- Unknown status -- " ) ;
      break ;
    }
  }

  protected static void printStateName ( int state ) {
    switch ( state ) {
    case Task.initialState:
      print ( "initial " ) ;
      break ;

    case Task.checkTimer:
      print ( "sleeping" ) ;
      break ;

    case Task.interrupt:
      print ( "interrupt" ) ;
      break ;

    case Task.terminate:
      print ( "terminate" ) ;
      break ;

    case Task.stopped:
      print ( "stopped  " ) ;
      break ;

    default:
      print ( state ) ;
      break ;
    }
  }

  /**
   * Print memory status
   *
   * @param stream output stream
   */
  public static void printMemoryStatus () {
    print   ( Memory.freeMemory ()) ;
    println ( " bytes free" ) ;
  }


  /**
   * Print task status
   */
  public static void printTaskStatus ( Task task ) {
    print   ( task ) ;
    printTaskStatus ( task.taskStatus ) ;
    printStateName ( task.state ) ;
    println () ;
  }

  /**
   * Print list of tasks
   *
   * @param prefix print before list of tasks
   * @param running true if running tasks should be listed
   */
  public static void printTasks ( String prefix, boolean running ) {
    println ( prefix ) ;

    for ( Task task = Task.taskStatusList
        ; task != null
        ; task = task.nextTaskStatus
        ) {
      if ( task.running () == running ) {
        print ( " " ) ;
        printTaskStatus ( task ) ;
      }
    }
  }

  /**
   * Generate a random number between 0 and limit
   *
   * @param stream output stream
   */
  public static void show () {
    Task task = Task.taskList ;

    println ( "== Task Status ==" ) ;
    printMemoryStatus () ;

    printTasks ( "Tasks running", true ) ;
    printTasks ( "Tasks not running", false ) ;

    if ( task == null ) {
      println ( "-- Empty task list --" ) ;
    } else {
      // Check taskList integrity
      do {
        task = task.nextTask ;
        if ( task == null ) {
          println ( "== Error: Task list not circular ==" ) ;
          break ;
        }
      } while ( Task.taskList != task ) ;
    }

    println () ;
  }
}