package stamp.util.os;

import stamp.core.*;
import stamp.util.os.*;

/**
 * Watch heap space task.
 * <p>
 * Start this task after all memory has been allocated.
 * It will track free memory and report any changes.
 *
 * @version 1.0 8/15/02
 *
 * @author Parallax Inc.
 */

public class WatchHeapTask extends Task {
  protected int     freeMemory = 0 ;
  protected boolean enable = true ;
  public    Event   event ;

  /**
   * Create WatchHeapTask.
   * Errors reported on System.out
   */
  public WatchHeapTask () {
  }


  /**
   * Control error checking
   */
  public void enable ( boolean enableNow ) {
    enable = enableNow ;

    if ( enableNow ) {
      freeMemory = 0 ;
    }
  }

  /**
   * Return task name
   */
  public String name () {
    return "WatchHeap" ;
  }

  /**
   * Create WatchHeapTask.
   * Errors reported by notifying the event.
   *
   * @param event Event to notify when heap grows
   */
  public WatchHeapTask ( Event event ) {
    this.event = event ;
  }

  protected void execute () {
    /* Memory.freeMemory is stack point dependent
     * so it must be called at the same point each
     * time to provide consistent information
     */
    int latestFreeMemory = Memory.freeMemory () ;

    // Exit if this is the only task running

    if ( nextTask == this ) {
      stop () ;
      return ;
    }

    if ( enable ) {
      if ( freeMemory == 0 ) {
        freeMemory = latestFreeMemory ;
      } else if ( freeMemory != latestFreeMemory ) {
        // Memory usage has increased (cannot decrease)

        if ( event == null ) {
          System.out.println ( "Error: Memory leak" ) ;
          System.out.print   ( " Last: " ) ;
          System.out.println ( freeMemory ) ;
          System.out.print   ( " Now:  " ) ;
          System.out.println ( latestFreeMemory ) ;
        } else {
          event.notify () ;
        }

        // Reset low water mark
        freeMemory = latestFreeMemory ;
      }
    }
  }
}