CON
  'Javelin IDE constants
  kReplyBit          = $80
  kFlag              = $7E
  kEscape            = $7D
  kInterrupt         = $19
  kResetWait         = 10000 '// Time taken for SX to reset.
  kAckWait           = 5000  '// The amount of time (in milliseconds) to wait for acknowledgement.
  kResetCharWait     = 3     '// The amount of time to wait before sending first char after a reset.
  kProgramWait       = 2000  '// The amount of time (in milliseconds) to wait for acknowledgement when programming.
  kStepWait          = 20000 '// The amount of time (in milliseconds) to wait for acknowledgement when stepping.
  kBaseVersion       = $50   '// Javelin compatible base version of the firmware.
  kMaxQueueLength    = 500   '// Maximum number of characters than can be queued.
  kComNone           = 0
  kComStatus         = 1
  kComReset          = 2
  kComStep           = 3
  kComRun            = 4
  kComError          = 5
  kComStop           = 6
  kComStack          = 7
  kComConfig         = 8
  kComHeap           = 9
  kComBreakpoints    = 10
  kComProgram        = 11
  kComDownloadOK     = 12
  kComStepDone       = 13
  kComAtn            = 14
  kComStartLoad      = 15
  kComEndLoad        = kBaseVersion
  kComTerminalQuery  = 17     '// Check for data from the terminal.
  kComTerminalData   = 18     '// Data follows.
  kComTerminalNoData = 19     '// No data follows.
  kComDownloadRamOK  = 20

  'values following kComError
  kErrNone       = 0
  kErrOpcode     = 1
  kErrBreakpoint = 2
  kErrMessage    = 3   'to send message to IDE message window
  kErrException  = 4   'to send exception to IDE status window
  kErrEnd        = 5   'to signal end of java program
  kErrJvm        = 6   'to send message to IDE status window

CON
  
  '//Spin stamp debug pin assignments
  stampSOUT = 16 'serial out (pin 1 of spin stamp)
  stampSIN  = 17 'serial in  (pin 2 of spin stamp)
  stampATN  = 18 'digital in (pin 3 of spin stamp, when activated, do reboot)

  '//Propeller system pin assignments
  propSCL = 28 'external eeprom SCL
  propSDA = 29 'external eeprom SDA
  propTX  = 30 'programming output
  propRX  = 31 'programming input

  BAUD    = 28800 '115200
  
CON

  '// The SX banks the VPs use.
  nmPeriph0Bank = $80
  nmPeriph1Bank = nmPeriph0Bank + $10
  nmPeriph2Bank = nmPeriph0Bank + $20
  nmPeriph3Bank = nmPeriph0Bank + $30
  nmPeriph4Bank = nmPeriph0Bank + $40
  nmPeriph5Bank = nmPeriph0Bank + $50
  nmBlockBank   = $70

  nmPortA       = $06  'corresponds to I/O pins 0..7
  nmPortB       = $07  'corresponds to I/O pins 8..15
  'reserved values for extending Javelin code to PropJava code
  nmPortC       = $08  'corresponds to I/O pins 16..23
  nmPortD       = $09  'corresponds to I/O pins 24..31
  
  pin0  = (1<<0)         'A constant representing pin P0
  pin1  = (1<<1)         'A constant representing pin P1
  pin2  = (1<<2)         'A constant representing pin P2
  pin3  = (1<<3)         'A constant representing pin P3 
  pin4  = (1<<4)         'A constant representing pin P4 
  pin5  = (1<<5)         'A constant representing pin P5
  pin6  = (1<<6)         'A constant representing pin P6 
  pin7  = (1<<7)         'A constant representing pin P7 
  pin8  = (1<<0) + $100  'A constant representing pin P8
  pin9  = (1<<1) + $100  'A constant representing pin P9
  pin10 = (1<<2) + $100  'A constant representing pin P10
  pin11 = (1<<3) + $100  'A constant representing pin P11 
  pin12 = (1<<4) + $100  'A constant representing pin P12 
  pin13 = (1<<5) + $100  'A constant representing pin P13
  pin14 = (1<<6) + $100  'A constant representing pin P14
  pin15 = (1<<7) + $100  'A constant representing pin P15  
  'reserved values for extending Javelin code to PropJava code
  pin16 = (1<<0) + $200  'A constant representing pin P16
  pin17 = (1<<1) + $200  'A constant representing pin P17
  pin18 = (1<<2) + $200  'A constant representing pin P18
  pin19 = (1<<3) + $200  'A constant representing pin P19 
  pin20 = (1<<4) + $200  'A constant representing pin P20 
  pin21 = (1<<5) + $200  'A constant representing pin P21
  pin22 = (1<<6) + $200  'A constant representing pin P22
  pin23 = (1<<7) + $200  'A constant representing pin P23  
  pin24 = (1<<0) + $300  'A constant representing pin P24
  pin25 = (1<<1) + $300  'A constant representing pin P25
  pin26 = (1<<2) + $300  'A constant representing pin P26
  pin27 = (1<<3) + $300  'A constant representing pin P27 
  pin28 = (1<<4) + $300  'A constant representing pin P28 
  pin29 = (1<<5) + $300  'A constant representing pin P29
  pin30 = (1<<6) + $300  'A constant representing pin P30
  pin31 = (1<<7) + $300  'A constant representing pin P31  

  'native registers

  'file stamp\core\EEPROM.java
  JVM_SB_MSB = $4C  '// Highbyte of stackbase (first free eeprom address)
  JVM_SB_LSB = $4D  '// Lowbyte of stackbase (first free eeprom address)
  {{
     The EEPROM.java methods size(),write() and read() use $7FFF for eeprom size.
     On the prop the eeprom size is set to JAVASIZE. To let the methods access the
     correct addresses, we must adjust for this difference. That means the value
     from register JVM_SB must be increased by JVM_SB_ADJUST. Originally JVM_SB is
     set to the stack base as found on eeprom address $0012.
  }}

  'file stamp\core\Memory.java
  JVM_HEAP_POINTER_LSB  = $18   '//jvm heap pointer (word)
  JVM_HEAP_POINTER_MSB  = $19
  JVM_STACK_POINTER_LSB = $12   '//jvm stack pointer (word)
  JVM_STACK_POINTER_MSB = $13

  {{
         Stackframe format
         +---------------+
         | Local0        | <- JVM_FP
         | ...           |
         | LocalN        |
         +---------------+
         | Method MSB    | <- JVM_FP + numLocals
         | Method LSB    |
         +---------------+
         | Old FP MSB    |
         | Old FP LSB    |
         +---------------+
         | JVM_PC MSB    |
         | JVM_PC LSB    |
         +---------------+
         | Stack0        |
         | ....          |
         +---------------+
                           <- JVM_SP
  }}

  kMPOffset = 0   '//method pointer offset
  kFPOffset = 2   '//frame pointer offset
  kPCOffset = 4   '//program counter offset

  {{
         Memory map layout
  $0000  +---------------+ @javaProg
         |    header     |
  $0014  +---------------+
         | jemcodes      |
         | ...           |
  $XXX0  +---------------+ stackbase                 ---
         | Local0        | <- JVM_FP                  ^
         | ...           |                            |
         | LocalN        |                            |
         +---------------+                            |
         | Method MSB    | <- JVM_FP + numLocals      |
         | Method LSB    |                            |
         +---------------+                            |
         | Old FP MSB    |                            |
         | Old FP LSB    |
         +---------------+                       stack frame
         | JVM_PC MSB    |                            
         | JVM_PC LSB    |                            | 
         +---------------+                            |
         | Stack0        | operand stack              |
         | ....          |                            V
         +---------------+                           ---
         |               | <- JVM_SP
         |     free      |
         |               |

         |               |
         |     free      |
         |               |
         +---------------+
         |               | <- JVM_HP
         |   allocated   |
         |               |
         +---------------+
         |  static vars  | size := byte[@javaProg + JVM_STATIC]   'size in bytes
         |  each var     | if size == 0 
         |    2 bytes    |   size := 256
         +---------------+                           
  JAVASIZE                 heapbase (on javelin $7FFF)

  Header layout:
  
  0000:   dw      $A4     ; Placeholder for run/debug.
  0001:   dw      $0      ; Placeholder for the length.
  0002:   dw      $01     ; Version: 0.1
  0003:   dw      16      ; Number of static fields = 8 (maximum of 128 static fields)
  0004:   dw      $00     ; `main' method: Offset hi = $00
  0005:   dw      $2C     ;                       lo = $2C
  0006:   dw      $00     ; java.lang.Object class: Offset hi = $00
  0007:   dw      $8D     ;                                lo = $8D
  0008:   dw      $01     ; java.lang.String class: Offset hi = $01
  0009:   dw      $59     ;                                lo = $59
  000A:   dw      $00     ; Throwable.throwVMException method: Offset hi = $00
  000B:   dw      $52     ;                                           lo = $52
  000C:   dw      $09     ; String table offset hi = $09
  000D:   dw      $7F     ;                     lo = $7F
  000E:   dw      $00     ; Callback method offset hi = $00
  000F:   dw      $00     ;                        lo = $00
  0010:   dw      $01     ; java.lang.OutOfMemoryError class: Offset hi = $01
  0011:   dw      $2C     ;                                          lo = $2C
  0012:   dw      $09     ; Stack base hi = $09
  0013:   dw      $B0     ;            lo = $B0
  
  }}

  'javaProg header fields
  JVM_RUNDEBUG     = $0000
  JVM_LENGTH       = $0001
  JVM_VERSION      = $0002
  JVM_STATIC       = $0003
  JVM_MAIN         = $0004
  JVM_OBJECT       = $0006
  JVM_STRING       = $0008
  JVM_THROW        = $000A
  JVM_STRTABLE     = $000C
  JVM_CALLBACK     = $000E
  JVM_MEMERROR     = $0010
  JVM_STKBASE      = $0012

  'class header fields
  JVM_SUPER        = $00
  JVM_FIELDS       = $02
  
  'method header fields
  JVM_PARAMETERS    = $00
  JVM_LOCALS        = $01
  JVM_FLAGS         = $02
  JVM_CODE          = $03
  JVM_EXCEPTION     = $05

  'file stamp\core\ADC.java (offsets within a VP bank) 
  nmAdcValue = $0B  '// Current ADC value.
  nmAdcInPort   = $0C  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmAdcInPin    = $0D  '// bitmask (1 of 8 set)
  nmAdcOutPort  = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmAdcOutPin   = $0F  '// bitmask (1 of 8 set)

  'unused registers used for accumulator, count and pin masks
  nmAdcCount   = $01
  nmAdcInMask  = $02  'long-aligned input pin mask
  nmAdcOutMask = $06  'long-aligned output pin mask
  nmAdcAccu    = $0A
  
  'file stamp\core\DAC.java (offsets within a VP bank) 
  nmDacValue = $0B  '// Current DAC value (dutycycle).
  nmDacPort  = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmDacPin   = $0F  '// bitmask (1 of 8 set)
  'unused registers used for accumulator
  nmDacAccu  = $0A

  'file stamp\core\PWM.java (offsets within a VP bank) 
  nmPWMPort     = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmPWMPin      = $0F  '// bitmask (1 of 8 set)
  'not defined in java sources but we can use unused registers
  'Peter's defines
  'read parameters as long
  nmPWMState    = $06  'state=0 or state=1 (long aligned)
  nmPWMCount    = $07  'subtract 256 from long to decrement count
  nmPWMhighTime = $08
  nmPWMlowTime  = $09
  nmPWMhighZero = $0A 'not used
  nmPWMlowZero  = $0B 'not used
  
  'file stamp\core\Timer.java (offsets within a VP bank) 
  nmPeriphISRVec = $00  '//
 'Peter's defines  
  nmLatchTimer1  = $06  '//32bit latch register, note that actual location is long aligned
  nmLatchTimer2  = $07
  nmLatchTimer3  = $08
  nmLatchTimer4  = $09  
  nmTimer1       = $02  '//32bit free running timer, incremented every 8.68 microsecond, long aligned
  nmTimer2       = $03
  nmTimer3       = $04
  nmTimer4       = $05

  'file stamp\core\Uart.java (offsets within a VP bank)
  nmUartByte     = $01  '// $01 free place for byte saved as start bit + bits + stop bit ... prop vars are long 
  nmUartDataH    = $02  '// transmit register highbyte
  nmUartDataL    = $03  '// transmit register lowbyte
  nmUartCnt      = $04  '// $04 free
  nmUartInvert   = $05  '// 0 for normal, 1 for invert
  nmUartBaud     = $06  '// 1000000/(bps * 4.34)
  nmUartStopBits = $07  '// Tx only: 00000001 for 1 stop bit, 00000011 for 2 etc
  nmUartHead     = $08  '//Index of the next free byte in the buffer. 
  nmUartTail     = $09  '//Index of the oldest byte in the buffer.
                        '//If tailPointer == headPointer the buffer is empty.
  nmUartBufferH  = $0A  '// address of 256 byte buffer (aligned to $XX00) 
  nmUartBufferL  = $0B
  nmUartPort     = $0C  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmUartPin      = $0D  '// bit mask (1 of 8 set)
  nmUartHSPort   = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmUartHSPin    = $0F  '// bitmask (1 of 8 set)

  'Uart baud constants
  speed600   = 192  'The speed value for 600 baud.
  speed1200  = 120  'The speed value for 1200 baud.
  speed2400  = 48   'The speed value for 2400 baud.
  speed4800  = 24   'The speed value for 4800 baud.
  speed7200  = 16   'The speed value for 7200 baud.
  speed9600  = 12   'The speed value for 9600 baud.
  speed14400 = 8    'The speed value for 14400 baud.
  speed19200 = 6    'The speed value for 19200 baud.
  speed28800 = 4    'The speed value for 28800 baud.
  speed38400 = 3    'The speed value for 38400 baud.
  speed57600 = 2    'The speed value for 57600 baud.


CON
  BREAKPOINTS = 10     'number of breakpoints supported by debugger
  MAX_NUM_VPS = 6      'Number of available VP slots
  MAX_COG_VPS = 2      'Max VPS per COG-1

CON
{{
/**
 * @enum VPTypeEnum
 * VP_NONE=0,  VP_TIMER, VP_ADC, VP_DAC, VP_PWM, VP_UART_RX, VP_UART_TX
 */
 _ typedef enum VPTypeEnum_t
}}
  #0, VP_NONE, VP_TIMER, VP_ADC, VP_DAC, VP_PWM, VP_UART_RX, VP_UART_TX


PUB dummy

