package stamp.core;

/**
 * This class provides constants and static methods
 * to support the use of odd and even parity.
 *
 * Date 10-22-2004
 * @author Peter Verkaik (peterverkaik@boselectro.nl)
 */

public class UartTools {

  //parity modes
  public static final int NONE = 0;  //no parity
  public static final int ODD  = 1;  //odd parity
  public static final int EVEN = 2;  //even parity

  /**
   * Test if character has even parity.
   * Ascii range is 0-127.
   *
   * @param data The character to test.
   *             Bit7 is the parity bit.
   * @return True if bit7 reflects even parity.
   */
  public static boolean isEvenParity(int data) {
    return evenParity(data) == data;
  }

  /**
   * Convert character to even parity character.
   * Ascii range is 0-127.
   *
   * @param data The character to convert.
   *             Bit7 is set to reflect even parity.
   * @return Even parity character.
   */
  public static int evenParity(int data) {
    int result=0;
    for (int i=1; i<8; i++) result ^= ((data<<i)&0x80);
    return ((data & 0x7F) | result);
  }

  /**
   * Test if character has odd parity.
   * Ascii range is 0-127.
   *
   * @param data The character to test.
   *             Bit7 is the parity bit.
   * @return True if bit7 reflects odd parity.
   */
  public static boolean isOddParity(int data) {
    return oddParity(data) == data;
  }

  /**
   * Convert character to odd parity character.
   * Ascii range is 0-127.
   *
   * @param data The character to convert.
   *             Bit7 is set to reflect odd parity.
   * @return Odd parity character.
   */
  public static int oddParity(int data) {
    return (0x80 ^ evenParity(data));
  }

}
