/*
 *
 * @file SLlist.java
 *
 * Defines an unsorted singly linked list Abstract Data Type
 *
 * Add performance is O(1)
 * Delete performance is O(N)
 * Find performance is O(N)
 *
 */
public class SLlist
{
  protected LLnode top;   // head of list
  protected LLnode end;   // end of list
  protected LLnode nextnode; // next in list iterator
  protected int size;

  public SLlist()
  {
    size = 0;
    top  = null;
    end  = null;
    nextnode = top;
  }


  /*
   * add node by reference
   */
  public void add(String string)
  {
    add((Object)string);
  }

  /*
   * add node by reference to end of list
   */
  public void add(Object obj)
  {
    LLnode node = new LLnode(obj);
    if(top == null) {
      top = node;
      end = node;
    }
    else {
      end.setNext(node);  // old end node points to new node
      end = node;         // new end node is new node
    }
    size++;
  }

  /*
   * delete node by reference
   */
  public void delete(LLnode obj)
  {
    LLnode node = null;
    LLnode next = top;

    //System.out.println("Delete '" + string + "'");
    if(top == null)
      return;

    if(obj == top) {
      if(end == top)
        end = top.getNext();  // delete end
      top = top.getNext();    // delete top
      size--;
      return;
    }

    while(next.getNext() != null) {
      if(obj == next.getNext()) {
        if(end == next)
          end = null;   // delete end
        next.setNext(next.getNext().getNext());
        size--;
        break;
      }
    }
  }

  /*
   * delete node by string value
   */
  public void delete(String string)
  {
    LLnode node = find(string);
    if(node != null)
      delete(node);
  }

  /*
   * find node by value
   */
  public LLnode find(String string)
  {
    LLnode next = top;
    while(next != null) {
      Object ob = next.getObject();
      if(string.equals((String)ob))
        return next;
      next = next.getNext();
    }
    return (LLnode)null;
  }

  /*
   * get SLlist length
   */
  public int length()
  {
    return size;
  }

  /*
   * get next if available
   */
  public LLnode hasNext()
  {
    LLnode node = nextnode;
    if(nextnode == null) {
      nextnode = top;
      return null;
    }
    nextnode = nextnode.getNext();
    return node;
  }

  /*
   * show list
   */
  public void show()
  {
    System.out.println("");
    System.out.println("Found " + length() + " Total Elements:");
    LLnode next = top;
    while(next != null) {
      Object ob = next.getObject();
      System.out.println((String)ob);
      next = next.getNext();
    }
  }
}

